@@ -6,9 +6,9 @@ use std::fmt;
 use async_trait::async_trait;
 use destream::{de, en};
 use futures::future::TryFutureExt;
+use log::debug;
 use safecast::{TryCastFrom, TryCastInto};
-
-use tc_btree::{BTreeType, Node};
+use tc_btree::BTreeType;
 use tc_error::*;
 use tc_transact::fs::{Dir, File, Persist, Restore, Store};
 use tc_transact::{IntoView, Transact, Transaction, TxnId};
@@ -18,7 +18,6 @@ use crate::collection::{
     BTree, BTreeFile, Collection, CollectionType, Table, TableIndex, TableType,
 };
 use crate::fs;
-use crate::route::Public;
 use crate::scalar::{Link, OpRef, Scalar, TCRef, Value, ValueType};
 use crate::state::{State, StateView};
 use crate::txn::Txn;
@@ -29,6 +28,7 @@ pub use sync::SyncChain;
 
 mod block;
 mod data;
+
 mod sync;
 
 const BLOCK_SIZE: u64 = 1_000_000;
@@ -141,8 +141,6 @@ impl Subject {
                     .create_file(txn_id, SUBJECT.into(), BTreeType::default())
                     .await?;
 
-                let file = fs::File::<Node>::try_from(file)?;
-
                 BTreeFile::create(file, schema, txn_id)
                     .map_ok(Self::BTree)
                     .await
@@ -153,12 +151,10 @@ impl Subject {
                     .await
             }
             Schema::Value(value) => {
-                let file = dir
+                let file: fs::File<Value> = dir
                     .create_file(txn_id, SUBJECT.into(), value.class())
                     .await?;
 
-                let file = fs::File::<Value>::try_from(file)?;
-
                 file.create_block(txn_id, SUBJECT.into(), value.clone())
                     .await?;
 
@@ -171,9 +167,7 @@ impl Subject {
         match schema {
             Schema::BTree(schema) => {
                 if let Some(file) = dir.get_file(txn.id(), &SUBJECT.into()).await? {
-                    BTreeFile::load(txn, schema, file.try_into()?)
-                        .map_ok(Self::BTree)
-                        .await
+                    BTreeFile::load(txn, schema, file).map_ok(Self::BTree).await
                 } else {
                     Self::create(Schema::BTree(schema), dir, *txn.id()).await
                 }
@@ -189,7 +183,7 @@ impl Subject {
             }
             Schema::Value(value) => {
                 if let Some(file) = dir.get_file(txn.id(), &SUBJECT.into()).await? {
-                    file.try_into().map(Self::Value)
+                    Ok(Self::Value(file))
                 } else {
                     Self::create(Schema::Value(value), dir, *txn.id()).await
                 }
@@ -219,17 +213,6 @@ impl Subject {
             }
         }
     }
-
-    async fn apply(&self, txn: &Txn, mutation: &data::Mutation) -> TCResult<()> {
-        use data::Mutation;
-
-        match mutation {
-            Mutation::Delete(path, key) => self.delete(txn, path, key.clone()).await,
-            Mutation::Put(path, key, value) => {
-                self.put(txn, path, key.clone(), value.clone().into()).await
-            }
-        }
-    }
 }
 
 #[async_trait]
@@ -258,13 +241,12 @@ impl de::FromStream for Subject {
     async fn from_stream<D: de::Decoder>(txn: Txn, decoder: &mut D) -> Result<Self, D::Error> {
         let value = Value::from_stream((), decoder).await?;
 
-        let file: fs::FileEntry = txn
+        let file: fs::File<Value> = txn
             .context()
             .create_file(*txn.id(), SUBJECT.into(), value.class())
             .map_err(de::Error::custom)
             .await?;
 
-        let file: fs::File<Value> = file.try_into().map_err(de::Error::custom)?;
         file.create_block(*txn.id(), SUBJECT.into(), value)
             .map_err(de::Error::custom)
             .await?;
@@ -312,7 +294,7 @@ pub trait ChainInstance {
         txn_id: TxnId,
         path: TCPathBuf,
         key: Value,
-        value: Scalar,
+        value: State,
     ) -> TCResult<()>;
 
     async fn last_commit(&self, txn_id: TxnId) -> TCResult<Option<TxnId>>;
@@ -323,8 +305,7 @@ pub trait ChainInstance {
     /// Replicate this [`Chain`] from the [`Chain`] at the given [`Link`].
     async fn replicate(&self, txn: &Txn, source: Link) -> TCResult<()>;
 
-    /// Prepare to commit by syncing this transaction's mutations to disk.
-    async fn prepare_commit(&self, txn_id: &TxnId);
+    async fn write_ahead(&self, txn_id: &TxnId);
 }
 
 /// The type of a [`Chain`].
@@ -334,6 +315,12 @@ pub enum ChainType {
     Sync,
 }
 
+impl Default for ChainType {
+    fn default() -> Self {
+        Self::Sync
+    }
+}
+
 impl Class for ChainType {}
 
 impl NativeClass for ChainType {
@@ -400,7 +387,7 @@ impl ChainInstance for Chain {
         txn_id: TxnId,
         path: TCPathBuf,
         key: Value,
-        value: Scalar,
+        value: State,
     ) -> TCResult<()> {
         match self {
             Self::Block(chain) => chain.append_put(txn_id, path, key, value).await,
@@ -429,10 +416,12 @@ impl ChainInstance for Chain {
         }
     }
 
-    async fn prepare_commit(&self, txn_id: &TxnId) {
+    async fn write_ahead(&self, txn_id: &TxnId) {
+        debug!("Chain::write_ahead {}", txn_id);
+
         match self {
-            Self::Block(chain) => chain.prepare_commit(txn_id).await,
-            Self::Sync(chain) => chain.prepare_commit(txn_id).await,
+            Self::Block(chain) => chain.write_ahead(txn_id).await,
+            Self::Sync(chain) => chain.write_ahead(txn_id).await,
         }
     }
 }
@@ -494,7 +483,7 @@ impl fmt::Display for Chain {
 
 /// A helper struct for [`ChainView`]
 pub enum ChainViewData<'en> {
-    Block((Schema, block::BlockSeq)),
+    Block((Schema, data::HistoryView<'en>)),
     Sync(Box<(Schema, StateView<'en>)>),
 }
 
