@@ -1,26 +1,23 @@
 //! A [`super::Chain`] which keeps only the data needed to recover the state of its subject in the
 //! event of a transaction failure.
 
-use std::convert::{TryFrom, TryInto};
-
 use async_trait::async_trait;
 use destream::de;
 use futures::future::TryFutureExt;
 use futures::join;
 
 use tc_error::*;
-use tc_transact::fs::{Dir, File, Persist};
+use tc_transact::fs::{Persist, Store};
 use tc_transact::{IntoView, Transact, Transaction, TxnId};
-use tcgeneric::{label, Label, TCPathBuf};
+use tcgeneric::TCPathBuf;
 
 use crate::fs;
-use crate::scalar::{Link, Scalar, Value};
-use crate::state::StateView;
+use crate::scalar::{Link, Value};
+use crate::state::{State, StateView};
 use crate::txn::Txn;
 
-use super::{ChainBlock, ChainInstance, ChainType, Schema, Subject, CHAIN, NULL_HASH};
-
-const BLOCK_ID: Label = label("sync");
+use super::data::History;
+use super::{ChainBlock, ChainInstance, ChainType, Schema, Subject, NULL_HASH};
 
 /// A [`super::Chain`] which keeps only the data needed to recover the state of its subject in the
 /// event of a transaction failure.
@@ -28,13 +25,14 @@ const BLOCK_ID: Label = label("sync");
 pub struct SyncChain {
     schema: Schema,
     subject: Subject,
-    file: fs::File<ChainBlock>,
+    history: History,
 }
 
 #[async_trait]
 impl ChainInstance for SyncChain {
     async fn append_delete(&self, txn_id: TxnId, path: TCPathBuf, key: Value) -> TCResult<()> {
-        let mut block = self.file.write_block(txn_id, BLOCK_ID.into()).await?;
+        let mut block = self.history.write_latest(txn_id).await?;
+        block.clear_until(&txn_id);
         block.append_delete(txn_id, path, key);
         Ok(())
     }
@@ -44,16 +42,18 @@ impl ChainInstance for SyncChain {
         txn_id: TxnId,
         path: TCPathBuf,
         key: Value,
-        value: Scalar,
+        value: State,
     ) -> TCResult<()> {
-        let mut block = self.file.write_block(txn_id, BLOCK_ID.into()).await?;
-        block.append_put(txn_id, path, key, value);
-        Ok(())
+        {
+            let mut block = self.history.write_latest(txn_id).await?;
+            block.clear_until(&txn_id);
+        }
+
+        self.history.append_put(txn_id, path, key, value).await
     }
 
     async fn last_commit(&self, txn_id: TxnId) -> TCResult<Option<TxnId>> {
-        let block = self.file.read_block(txn_id, BLOCK_ID.into()).await?;
-        Ok(block.mutations().keys().next().cloned())
+        self.history.last_commit(txn_id).await
     }
 
     fn subject(&self) -> &Subject {
@@ -64,17 +64,14 @@ impl ChainInstance for SyncChain {
         let subject = txn.get(source, Value::None).await?;
         self.subject.restore(*txn.id(), subject).await?;
 
-        let mut block = self.file.write_block(*txn.id(), BLOCK_ID.into()).await?;
+        let mut block = self.history.write_latest(*txn.id()).await?;
         *block = ChainBlock::with_txn(NULL_HASH, *txn.id());
 
         Ok(())
     }
 
-    async fn prepare_commit(&self, txn_id: &TxnId) {
-        self.file
-            .sync_block(*txn_id, BLOCK_ID.into())
-            .await
-            .expect("prepare SyncChain commit");
+    async fn write_ahead(&self, txn_id: &TxnId) {
+        self.history.commit(txn_id).await
     }
 }
 
@@ -88,56 +85,30 @@ impl Persist<fs::Dir, Txn> for SyncChain {
     }
 
     async fn load(txn: &Txn, schema: Self::Schema, dir: fs::Dir) -> TCResult<Self> {
+        let is_new = dir.is_empty(txn.id()).await?;
+
         let subject = Subject::load(txn, schema.clone(), &dir).await?;
 
-        let txn_id = *txn.id();
-        let file = if let Some(file) = dir.get_file(&txn_id, &CHAIN.into()).await? {
-            let file = fs::File::<ChainBlock>::try_from(file)?;
-
-            let block = file.read_block(txn_id, BLOCK_ID.into()).await?;
-            if block.mutations().len() > 1 {
-                return Err(TCError::internal(
-                    "SyncChain should only store one Transaction record at a time",
-                ));
-            }
-
-            if let Some((last_txn_id, ops)) = block.mutations().iter().next() {
-                for op in ops {
-                    subject
-                        .apply(txn, op)
-                        .map_err(|e| {
-                            TCError::internal(format!(
-                                "error replaying last transaction {}: {}",
-                                last_txn_id, e
-                            ))
-                        })
-                        .await?;
-                }
-            }
-
-            file
+        let history = if is_new {
+            History::create(*txn.id(), dir, ChainType::Sync).await?
         } else {
-            let file = dir
-                .create_file(txn_id, CHAIN.into(), ChainType::Sync)
-                .await?;
-
-            let file = fs::File::<ChainBlock>::try_from(file)?;
-            if !file.contains_block(&txn_id, &BLOCK_ID.into()).await? {
-                file.create_block(
-                    txn_id,
-                    BLOCK_ID.into(),
-                    ChainBlock::with_txn(NULL_HASH, txn_id),
-                )
-                .await?;
-            }
-
-            file
+            History::load(txn, (), dir).await?
         };
 
+        let latest = history.latest_block_id(txn.id()).await?;
+        if latest > 0 {
+            return Err(TCError::internal(format!(
+                "a SyncChain can only have one block, found {}",
+                latest
+            )));
+        }
+
+        history.apply_last(txn, &subject).await?;
+
         Ok(SyncChain {
             schema,
             subject,
-            file,
+            history,
         })
     }
 }
@@ -145,23 +116,11 @@ impl Persist<fs::Dir, Txn> for SyncChain {
 #[async_trait]
 impl Transact for SyncChain {
     async fn commit(&self, txn_id: &TxnId) {
-        {
-            let mut block = self
-                .file
-                .sync_block(*txn_id, BLOCK_ID.into())
-                .await
-                .unwrap();
-
-            self.subject.commit(txn_id).await;
-
-            *block = ChainBlock::with_txn(NULL_HASH, *txn_id);
-        }
-
-        self.file.commit(txn_id).await
+        self.subject.commit(txn_id).await;
     }
 
     async fn finalize(&self, txn_id: &TxnId) {
-        join!(self.subject.finalize(txn_id), self.file.finalize(txn_id));
+        join!(self.subject.finalize(txn_id), self.history.finalize(txn_id));
     }
 }
 
@@ -188,10 +147,7 @@ impl de::Visitor for ChainVisitor {
     }
 
     async fn visit_seq<A: de::SeqAccess>(self, mut seq: A) -> Result<Self::Value, A::Error> {
-        let file = self
-            .txn
-            .context()
-            .create_file(*self.txn.id(), CHAIN.into(), ChainType::Sync)
+        let history = History::create(*self.txn.id(), self.txn.context().clone(), ChainType::Sync)
             .map_err(de::Error::custom)
             .await?;
 
@@ -208,7 +164,7 @@ impl de::Visitor for ChainVisitor {
         Ok(SyncChain {
             schema,
             subject,
-            file: file.try_into().map_err(de::Error::custom)?,
+            history,
         })
     }
 }
