@@ -30,6 +30,8 @@ fn data_size(flag: &str) -> TCResult<u64> {
         Ok(size * 1000)
     } else if flag.ends_with('M') {
         Ok(size * 1_000_000)
+    } else if flag.ends_with('G') {
+        Ok(size * 1_000_000_000)
     } else {
         Err(TCError::bad_request(ERR, flag))
     }
@@ -60,7 +62,7 @@ struct Config {
     )]
     pub workspace: PathBuf,
 
-    #[structopt(long = "cache_size", default_value = "5K", parse(try_from_str = data_size))]
+    #[structopt(long = "cache_size", default_value = "10M", parse(try_from_str = data_size))]
     pub cache_size: u64,
 
     #[structopt(
