@@ -1,6 +1,6 @@
 //! Immutable values which always reside entirely in memory
 
-use std::collections::{HashMap, HashSet};
+use std::collections::{BTreeMap, HashSet};
 use std::convert::TryFrom;
 use std::fmt;
 use std::iter::FromIterator;
@@ -8,6 +8,7 @@ use std::ops::{Deref, DerefMut};
 use std::str::FromStr;
 
 use async_trait::async_trait;
+use bytes::Bytes;
 use destream::{de, en};
 use futures::future::{try_join_all, TryFutureExt};
 use log::debug;
@@ -315,6 +316,12 @@ impl From<Map<Scalar>> for Scalar {
     }
 }
 
+impl From<OpRef> for Scalar {
+    fn from(op_ref: OpRef) -> Self {
+        Self::Ref(Box::new(TCRef::Op(op_ref)))
+    }
+}
+
 impl From<Range> for Scalar {
     fn from(range: Range) -> Self {
         Self::Range(range)
@@ -384,6 +391,22 @@ impl TryFrom<Scalar> for Value {
     }
 }
 
+impl TryCastFrom<Scalar> for Bytes {
+    fn can_cast_from(scalar: &Scalar) -> bool {
+        match scalar {
+            Scalar::Value(value) => Self::can_cast_from(value),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(scalar: Scalar) -> Option<Self> {
+        match scalar {
+            Scalar::Value(value) => Self::opt_cast_from(value),
+            _ => None,
+        }
+    }
+}
+
 impl TryCastFrom<Scalar> for OpDef {
     fn can_cast_from(scalar: &Scalar) -> bool {
         match scalar {
@@ -441,8 +464,8 @@ impl TryCastFrom<Scalar> for Bound {
 
     fn opt_cast_from(scalar: Scalar) -> Option<Self> {
         match scalar {
-            Scalar::Map(map) if HashMap::<Id, Value>::can_cast_from(&map) => {
-                let map = HashMap::<Id, Value>::opt_cast_from(map).unwrap();
+            Scalar::Map(map) if BTreeMap::<Id, Value>::can_cast_from(&map) => {
+                let map = BTreeMap::<Id, Value>::opt_cast_from(map).unwrap();
                 Map::from(map).opt_cast_into()
             }
             Scalar::Value(value) if value.is_none() => Some(Self::Un),
@@ -555,7 +578,7 @@ impl TryCastFrom<Scalar> for Number {
 impl<T: Clone + TryCastFrom<Scalar>> TryCastFrom<Scalar> for Map<T> {
     fn can_cast_from(scalar: &Scalar) -> bool {
         match scalar {
-            Scalar::Map(map) => HashMap::<Id, T>::can_cast_from(map),
+            Scalar::Map(map) => BTreeMap::<Id, T>::can_cast_from(map),
             Scalar::Tuple(tuple) => Vec::<(Id, T)>::can_cast_from(tuple),
             _ => false,
         }
@@ -563,7 +586,7 @@ impl<T: Clone + TryCastFrom<Scalar>> TryCastFrom<Scalar> for Map<T> {
 
     fn opt_cast_from(scalar: Scalar) -> Option<Self> {
         match scalar {
-            Scalar::Map(map) => HashMap::<Id, T>::opt_cast_from(map).map(Map::from),
+            Scalar::Map(map) => BTreeMap::<Id, T>::opt_cast_from(map).map(Map::from),
             Scalar::Tuple(tuple) => {
                 if let Some(entries) = Vec::<(Id, T)>::opt_cast_from(tuple) {
                     Some(entries.into_iter().collect())
@@ -905,10 +928,12 @@ impl de::Visitor for ScalarVisitor {
             return Ok(Scalar::Map(Map::default()));
         };
 
-        if let Ok(path) = TCPathBuf::from_str(&key) {
-            if let Some(class) = ScalarType::from_path(&path) {
-                if let Ok(scalar) = Self::visit_map_value(class, &mut access).await {
-                    return Ok(scalar);
+        if key.starts_with('/') {
+            if let Ok(path) = TCPathBuf::from_str(&key) {
+                if let Some(class) = ScalarType::from_path(&path) {
+                    if let Ok(scalar) = Self::visit_map_value(class, &mut access).await {
+                        return Ok(scalar);
+                    }
                 }
             }
         }
@@ -918,7 +943,7 @@ impl de::Visitor for ScalarVisitor {
             return Self::visit_subject(subject, params);
         }
 
-        let mut map = HashMap::new();
+        let mut map = BTreeMap::new();
         let key = Id::from_str(&key).map_err(de::Error::custom)?;
         let value = access.next_value(()).await?;
         map.insert(key, value);
