@@ -1,12 +1,13 @@
 //! A Tinychain [`State`]
 
-use std::collections::{HashMap, HashSet};
+use std::collections::{BTreeMap, HashSet};
 use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::iter::FromIterator;
 use std::str::FromStr;
 
 use async_trait::async_trait;
+use bytes::Bytes;
 use destream::de;
 use futures::future::try_join_all;
 use futures::TryFutureExt;
@@ -213,7 +214,7 @@ impl Refer for State {
                 )
                 .await?;
 
-                let map = HashMap::from_iter(resolved);
+                let map = BTreeMap::from_iter(resolved);
                 Ok(State::Map(map.into()))
             }
             Self::Scalar(scalar) => scalar.resolve(context, txn).await,
@@ -390,10 +391,26 @@ impl TryCastFrom<State> for bool {
     }
 }
 
+impl TryCastFrom<State> for Bytes {
+    fn can_cast_from(state: &State) -> bool {
+        match state {
+            State::Scalar(scalar) => Self::can_cast_from(scalar),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(state: State) -> Option<Self> {
+        match state {
+            State::Scalar(scalar) => Self::opt_cast_from(scalar),
+            _ => None,
+        }
+    }
+}
+
 impl<T: Clone + TryCastFrom<State>> TryCastFrom<State> for Map<T> {
     fn can_cast_from(state: &State) -> bool {
         match state {
-            State::Map(map) => HashMap::<Id, T>::can_cast_from(map),
+            State::Map(map) => BTreeMap::<Id, T>::can_cast_from(map),
             State::Tuple(tuple) => Map::<T>::can_cast_from(tuple),
             _ => false,
         }
@@ -401,7 +418,7 @@ impl<T: Clone + TryCastFrom<State>> TryCastFrom<State> for Map<T> {
 
     fn opt_cast_from(state: State) -> Option<Self> {
         match state {
-            State::Map(map) => HashMap::<Id, T>::opt_cast_from(map).map(Map::from),
+            State::Map(map) => BTreeMap::<Id, T>::opt_cast_from(map).map(Map::from),
             State::Tuple(tuple) => Map::<T>::opt_cast_from(tuple),
             _ => None,
         }
@@ -573,7 +590,7 @@ impl TryCastFrom<State> for OpRef {
 impl TryCastFrom<State> for Scalar {
     fn can_cast_from(state: &State) -> bool {
         match state {
-            State::Map(map) => HashMap::<Id, Scalar>::can_cast_from(map),
+            State::Map(map) => BTreeMap::<Id, Scalar>::can_cast_from(map),
             State::Scalar(_) => true,
             State::Tuple(tuple) => Vec::<Scalar>::can_cast_from(tuple),
             _ => false,
@@ -582,7 +599,7 @@ impl TryCastFrom<State> for Scalar {
 
     fn opt_cast_from(state: State) -> Option<Self> {
         match state {
-            State::Map(map) => HashMap::<Id, Scalar>::opt_cast_from(map)
+            State::Map(map) => BTreeMap::<Id, Scalar>::opt_cast_from(map)
                 .map(Map::from)
                 .map(Scalar::Map),
 
@@ -768,13 +785,15 @@ impl<'a> de::Visitor for StateVisitor {
         if let Some(key) = access.next_key::<String>(()).await? {
             debug!("deserialize: key is {}", key);
 
-            if let Ok(path) = TCPathBuf::from_str(&key) {
-                debug!("is {} a classpath?", path);
-                if let Some(class) = StateType::from_path(&path) {
-                    debug!("deserialize instance of {}...", class);
-                    return self.visit_map_value(class, &mut access).await;
-                } else {
-                    debug!("not a classpath: {}", path);
+            if key.starts_with('/') {
+                if let Ok(path) = TCPathBuf::from_str(&key) {
+                    debug!("is {} a classpath?", path);
+                    if let Some(class) = StateType::from_path(&path) {
+                        debug!("deserialize instance of {}...", class);
+                        return self.visit_map_value(class, &mut access).await;
+                    } else {
+                        debug!("not a classpath: {}", path);
+                    }
                 }
             }
 
@@ -784,7 +803,7 @@ impl<'a> de::Visitor for StateVisitor {
                 return ScalarVisitor::visit_subject(subject, params).map(State::Scalar);
             }
 
-            let mut map = HashMap::new();
+            let mut map = BTreeMap::new();
 
             let id = Id::from_str(&key).map_err(de::Error::custom)?;
             let txn = self
